#define WIN32_LEAN_AND_MEAN
#define STRICT

#include <windows.h>
#include <stdio.h>
#include "lstetris.h"
#include "game.h"
#include "lsapi.h"

//there really should be a struct to hold all the game variables, but I'm lazy

HWND parent, hMainWnd;
char *szAppName = "LSTetris";
game g;
POINT winPos, winSize;
COLORREF back_color, text_color;
HBRUSH hbr_back;
HBITMAP backImage;
HFONT font;
int timer, clearing, rotate, speed;
extern int rows, cols, block_size;
extern int brickColors[][3];
BOOL show, clearAnimation, fixed, AlwaysOnTop, transparent;

LRESULT CALLBACK WndProc(HWND hwnd, UINT message, WPARAM wParam, LPARAM lParam)
{
	switch (message)
	{
	case WM_CREATE:
	case WM_PAINT:
		{
			if (!backImage && transparent==TRUE) // transparent
			{
				HDC hdc = GetDC(parent);
				HDC buf = CreateCompatibleDC(NULL);

				backImage = CreateCompatibleBitmap(hdc, winSize.x, winSize.y);
				SelectObject(buf, backImage);
				BitBlt(buf, 0, 0, winSize.x, winSize.y, 
					hdc, winPos.x-3, winPos.y-19, SRCCOPY);

				ReleaseDC(hwnd,hdc);
				DeleteDC(buf);
				DeleteDC(hdc);
			}

			PAINTSTRUCT ps;
			HDC hdc = BeginPaint( hwnd, &ps );
			HDC buf = CreateCompatibleDC(NULL);
			HDC src = CreateCompatibleDC(NULL);
			HBITMAP bufBMP = CreateCompatibleBitmap(hdc, winSize.x, winSize.y);
			SelectObject(buf, bufBMP);
			RECT r;
			char temp[64];

			r.bottom = winSize.y;
			r.top = r.left = 0;
			r.right = winSize.x;

			// draw background
			if (transparent==TRUE) // transparent
			{
				SelectObject(src, backImage);
				BitBlt(buf, 0, 0, winSize.x, winSize.y, src, 0, 0, SRCCOPY);
				SelectObject(buf, bufBMP);
			}
			else // setback color
			{
				FillRect(buf, &r, hbr_back);
			}

			// draw cur piece and board if game is running
			g.drawBoard(buf);
			if (g.getAlive() && !g.getPaused())
				g.drawCurPiece(buf);
			else
			{
				SetBkMode(buf, TRANSPARENT);
				SetTextColor(buf, text_color);
				SelectObject(buf, font);
				sprintf(temp, "Lines: %d\nGame Over\n\nPress enter to\nplay again", g.getLines());
				DrawText(buf, temp, -1,	&r, DT_CENTER);
			}

			BitBlt(hdc, 0, 0, winSize.x, winSize.y, buf, 0, 0, SRCCOPY);
			EndPaint( hwnd, &ps );
			DeleteDC(buf);
			DeleteDC(hdc);
			DeleteDC(src);
			DeleteObject(bufBMP);
			return 0;
		}
	case WM_KEYDOWN:
		{
			if (g.getAlive() && !g.getClearing())
			{
				if (wParam==VK_DOWN || wParam==VK_NUMPAD2) 
				{
					g.downPiece();
				}
				else if (wParam==VK_RIGHT || wParam==VK_NUMPAD6)
				{
					g.moveRight();
				}
				else if (wParam==VK_LEFT || wParam==VK_NUMPAD4)
				{
					g.moveLeft();
				}
				else if (wParam==VK_UP || wParam==VK_CONTROL || wParam==VK_NUMPAD8)
				{
					g.rotatePiece(rotate);
				}
	//			else if (wParam==VK_ALT)
	//			{
	//				g.rotatePiece(-1);
	//			}
				else if (wParam==VK_SPACE || wParam==VK_NUMPAD0)
				{
					g.dropPiece();
				}
			}
			
			if (wParam==VK_RETURN)
			{
				g.resetGame();
				timer = SetTimer(hwnd, 1, speed, NULL);
			}
			InvalidateRect(hwnd, NULL, TRUE);
			return 0;
		}
	case WM_LBUTTONDOWN:
		{
			if (fixed == FALSE)
				SendMessage( hwnd, WM_SYSCOMMAND, SC_MOVE | 2, lParam );
			return 0;
		}
	case WM_TIMER:
		{
			if (wParam==1)
			{
				if (g.getClearing())
				{
					KillTimer(hwnd, timer);
					clearing = SetTimer(hwnd, 2, 30, NULL);
				}
				else if (!g.getAlive())
				{
					KillTimer(hwnd, timer);
				}
				else
					g.downPiece();
				InvalidateRect(hwnd, NULL, TRUE);
			}
			else if (wParam==2)
			{
				if (g.getClearing())
				{
					if (clearAnimation==TRUE)
						g.clearLineAnimation();
					else
						g.clearAllLines();
					InvalidateRect(hwnd, NULL, TRUE);
				}
				else
				{
					KillTimer(hwnd,clearing);
					timer = SetTimer(hwnd, 1, speed, NULL);
				}
			}
			return 0;
		}
	default:
		return DefWindowProc(hwnd,message,wParam,lParam);
	}

	return DefWindowProc(hwnd,message,wParam,lParam);
}

void showWnd(HWND hwnd, char *args)
{
	ShowWindow(hMainWnd, SW_SHOWNORMAL);
	show = TRUE;
}

void hideWnd(HWND hwnd, char *args)
{
	ShowWindow(hMainWnd, SW_HIDE);
	show = FALSE;
}

void toggleWnd(HWND hwnd, char *args)
{
	if (!show)
		showWnd(hwnd, args);
	else
		hideWnd(hwnd, args);
}

void focusWnd(HWND hwnd, char *args)
{
	showWnd(hwnd, args);
	SetForegroundWindow(hMainWnd);
	SetFocus(hMainWnd);
}

int initModule(HWND ParentWnd, HINSTANCE dllInst, wharfDataType* wd)
{
  return initModuleEx (ParentWnd, dllInst, wd->lsPath);
}

int initModuleEx(HWND ParentWnd, HINSTANCE dllInst, LPCSTR szPath)
{
	int font_size;
	parent = ParentWnd;
	
    WNDCLASS wc;

	AddBangCommand("!LSTetrisShow", showWnd);
	AddBangCommand("!LSTetrisHide", hideWnd);
	AddBangCommand("!LSTetrisToggle", toggleWnd);
	AddBangCommand("!LSTetrisFocus", focusWnd);
	block_size = GetRCInt("LSTetrisBlockSize", 5);
	winPos.x = GetRCInt("LSTetrisX", 100);
	winPos.y = GetRCInt("LSTetrisY", 100);
	winSize.x = GetRCInt("LSTetrisWidth", 60);
	winSize.y = GetRCInt("LSTetrisHeight", 110);
	show = GetRCBool("LSTetrisStartHidden", FALSE);
	font_size = GetRCInt("LSTetrisTextSize", 12);
	clearAnimation = GetRCBool("LSTetrisFadeLines", TRUE);
	fixed = GetRCBool("LSTetrisFixed", TRUE);
	AlwaysOnTop = GetRCBool("LSTetrisAlwaysOnTop", TRUE);
	transparent = GetRCBool("LSTetrisTransparent", TRUE);
	speed = GetRCInt("LSTetrisSpeed", 500);
	if (GetRCBool("LSTetrisRotateClockwise", FALSE))
		rotate = -1;
	else
		rotate = 1;
	font = CreateFont(font_size, 0, 0, 0, 0, FALSE, FALSE, FALSE,
		DEFAULT_CHARSET, OUT_DEFAULT_PRECIS, CLIP_DEFAULT_PRECIS,
		DEFAULT_QUALITY, DEFAULT_PITCH, NULL);

	back_color = GetRCColor("LSTetrisBackColor", RGB(0, 0, 0));
	text_color = GetRCColor("LSTetrisTextColor", RGB(192, 192, 192));
	hbr_back = CreateSolidBrush(back_color);
	//brickColors[0]

	// safety check
	if (winSize.x <= 0)
		winSize.x = 60;
	if (winSize.y <= 0)
		winSize.y = 110;

	// negative position coordinates
	RECT r;
	GetClientRect(GetDesktopWindow(),&r);

	if (winPos.x < 0)
		winPos.x = r.right + winPos.x;
	if (winPos.y < 0)
		winPos.y = r.bottom + winPos.y;


	cols = winSize.x / block_size;
	rows = winSize.y / block_size;

	g.togglePaused();

	memset(&wc,0,sizeof(wc));
    wc.lpfnWndProc = WndProc;       // our window procedure
    wc.hInstance = dllInst;         // hInstance of DLL
    wc.lpszClassName = szAppName;   // our window class name

    if (!RegisterClass(&wc)) 
    {
        MessageBox(parent,"Error registering window class",szAppName, MB_OK);
        return 1;
    }

    hMainWnd = CreateWindowEx(
        ((AlwaysOnTop==TRUE)?WS_EX_TOPMOST:0)|WS_EX_TOOLWINDOW,  // exstyles 
        szAppName,                                  // our window class name
        szAppName,                                  // use description for a window title
        WS_POPUP,                                   // window style
        winPos.x, winPos.y,							// position
        winSize.x, winSize.y,                       // width & height of window
        GetDesktopWindow(),                         // parent window (desktop)
        NULL,                                       // no menu
        dllInst,                                    // hInstance of DLL
        0);                                         // no window creation data

	if (!hMainWnd)
	{
		MessageBox(NULL, "Error creating window", szAppName, MB_OK);
		return 1;
	}

	if (show==TRUE)
		ShowWindow(hMainWnd, SW_SHOWNORMAL);
  return 0;
}

void quitModule(HINSTANCE dllInst)
{
	DeleteObject(hbr_back);
	DeleteObject(font);
	DeleteObject(backImage);
	DestroyWindow(hMainWnd);
	UnregisterClass(szAppName, dllInst);
	KillTimer(hMainWnd, clearing);
	KillTimer(hMainWnd, timer);
}
