#include "game.h"

int cols = 12;
int rows = 22;
int block_size = 16;

int brickColors[][3] = 
{ 
	{255, 255, 0},
	{100, 100, 255},
	{255, 255, 0},
	{0, 171, 0},
	{168, 0, 168},
	{50, 50, 255},
	{255, 0, 0} 
};

COLORREF darkRGB(int r, int g, int b)
{
	r -= 25;
	g -= 25;
	b -= 25;
	if (r < 0)
		r = 0;
	if (g < 0)
		g = 0;
	if (b < 0)
		b = 0;
	return RGB(r, g, b);
}

game::game()
{
// randomize the random number generator
	srand( (unsigned)time( NULL ) );
	next = true;
	grid = false;
	resetGame();
}

void game::resetGame()
{
	lines = 0;
	if (board)
	{
		delete board;
		board = NULL;
	}
	board = new BOARD [rows*cols];

	clearCol = 0;
	
	nextRot = rand() % NumPoses;
	nextPiece = rand() % NumBrickTypes;

	newPiece();

	paused = false;
	alive = true;

	for (int c = 0; c < cols; c++)
	{
		for (int r = 0; r < rows; r++)
		{
			boardGet(r, c)->filled = false;
		}
	}
}

game::~game()
{
	if (board)
	{
		delete board;
		board = NULL;
	}
}

BOARD *game::boardGet(int r, int c)
{
	return &board[r*cols + c];
}

// r is the amount of rotation
// -1 for clockwise, +1 for ccw
void game::rotatePiece(int r)
{
//	use some temp variables to see if rotation is possible
	int newCol = curCol;
	int newRow = curRow;
	int newRot = (curRot + r + 4) % 4;

	if (curCol + bricks[curPiece][newRot].w >= cols)
		newCol = cols - bricks[curPiece][newRot].w;
	if (curRow + bricks[curPiece][newRot].h >= rows)
		newRow = rows - bricks[curPiece][newRot].h;

	if (!collide(curPiece, newRot, newCol, newRow))
	{
		curRot = newRot;
		curCol = newCol;
		curRow = newRow;
	}
}


// draws the pieces that are currently on the board
void game::drawBoard(HDC hdc)
{
	RECT rect;
	HBRUSH hbr1;
	HBRUSH hbr2;
	for (int r = 0; r < rows; r++)
	{
		for (int c = 0; c < cols; c++)
		{
			if (boardGet(r, c)->filled)
			{
				hbr1 = CreateSolidBrush(darkRGB(boardGet(r,c)->color.r,
					boardGet(r,c)->color.g,
					boardGet(r,c)->color.b));
				hbr2  = CreateSolidBrush(RGB(boardGet(r,c)->color.r,
					boardGet(r,c)->color.g,
					boardGet(r,c)->color.b));
				
				rect.left = c*block_size;
				rect.top = ((rows-1)*block_size)-r * block_size;
				rect.right = rect.left + block_size;
				rect.bottom = rect.top + block_size;
				FillRect(hdc, &rect, hbr1);

				rect.left++;
				rect.top++;
				rect.right--;
				rect.bottom--;
				FillRect(hdc, &rect, hbr2);
				
				DeleteObject(hbr1);
				DeleteObject(hbr2);
			}
		}
	}
}

// moves the block down 1 row
void game::downPiece()
{
	if (!collide(curPiece, curRot, curCol, curRow-1))
		curRow--;
	else
		placePiece();
}

// places the piece on the board if it hits another piece
void game::placePiece()
{
	int numCleared = 0;

	for (int r = 0; r < bricks[curPiece][curRot].h; r++)
	{
		for (int c = 0; c < bricks[curPiece][curRot].w; c++)
		{
			if (bricks[curPiece][curRot].grid[r][c] == 'X')
			{
				boardGet(r+curRow, c+curCol)->filled = true;
				boardGet(r+curRow, c+curCol)->color.r = brickColors[curPiece][0];
				boardGet(r+curRow, c+curCol)->color.g = brickColors[curPiece][1];
				boardGet(r+curRow, c+curCol)->color.b = brickColors[curPiece][2];
			}
		}
	}

	numCleared = clearLines(curRow, bricks[curPiece][curRot].h);

//	if lines are cleared, new piece isn't determined until after animation
	if (!numCleared)
		newPiece();
}

// quick drop of a piece
void game::dropPiece()
{
	int iniRow = curRow;

	while (!collide(curPiece, curRot, curCol, curRow-1)) 
	{
		curRow--;
	}

	placePiece();
}

// after a piece is dropped, checks for complete rows and clears them
int game::clearLines(int bottomRow, int height)
{
	for (int r = 0; r < height; r++)
	{
		if (isFull(bottomRow+r))
			clearList.push_back(bottomRow+r);
	}

	if (clearList.size())
		clearing = true;
	else
		clearing = false;

	return clearList.size();
}

// test for complete row
bool game::isFull(int row)
{
	for (int c = 0; c < cols; c++)
	{
		if (!boardGet(row, c)->filled)
			return false;
	}
	return true;
}

void game::clearAllLines()
{
	list<int>::reverse_iterator li;

	for (li = clearList.rbegin(); li != clearList.rend(); li++)
	{
		for (int r = *li; r < rows-1; r++)
		{
			for (int c = 0; c < cols; c++)
			{
				*boardGet(r, c) = *boardGet(r+1, c);
			}
		}
	}
	clearing = false;
	clearCol = 0;
	lines += clearList.size();
	clearList.clear();

//	get new piece
	newPiece();
}

// fades full rows
void game::clearLineAnimation()
{
	list<int>::reverse_iterator li;

	bool done = false;
	for (li = clearList.rbegin(); li != clearList.rend(); li++)
	{
		for (int c = 0; c < clearCol; c++)
		{
			if (boardGet(*li, c)->color.fade() && c == cols - 1)
				done = true;
		}
	}

	if (clearCol < cols)
		clearCol++;

	if (done)
	{
		clearAllLines();
	}
}

// move the piece left
void game::moveLeft()
{
	if (!collide(curPiece, curRot, curCol-1, curRow))
		curCol--;
}

// move the piece right
void game::moveRight()
{
	if (!collide(curPiece, curRot, curCol+1, curRow))
		curCol++;
}

// all motion of a piece is tested to be valid (no collisions)
bool game::collide(int piece, int rot, int col, int row)
{
//	inside the walls
	if (col < 0 || col > cols - bricks[piece][rot].w)
		return true;
//	not off the bottom of the board
	if (row < 0)
		return true;

//	the piece itself and others on the board
	for (int r = 0; r < bricks[piece][rot].h; r++)
	{
		for (int c = 0; c < bricks[piece][rot].w; c++)
		{
			if (boardGet(r+row, c+col)->filled && 
				bricks[piece][rot].grid[r][c] == 'X')
				return true;
		}
	}
	return false;
}

// generates a new piece
void game::newPiece()
{
	curPiece = nextPiece;
	curRot = nextRot;
	curCol = (cols - bricks[curPiece][curRot].w) / 2;
	curRow = rows - bricks[curPiece][curRot].h;

	nextRot = rand() % NumPoses;
	nextPiece = rand() % NumBrickTypes;

	if (collide(curPiece, curRot, curCol, curRow))
		gameOver();
}

// game is over
void game::gameOver()
{
	alive = false;
}

// draws the current piece
void game::drawCurPiece(HDC hdc)
{
	POINT temp;
	temp.x = curCol * block_size;
	temp.y = curRow * block_size;
	drawPiece(hdc, curPiece, curRot, temp);
}

// draws any piece given position
void game::drawPiece(HDC &hdc, int piece, int rot, POINT offset)
{
	RECT rect;
	HBRUSH hbr1 = CreateSolidBrush(darkRGB(brickColors[piece][0],
		brickColors[piece][1],
		brickColors[piece][2]));
	HBRUSH hbr2 = CreateSolidBrush(RGB(brickColors[piece][0],
		brickColors[piece][1],
		brickColors[piece][2]));

	for (int r = 0; r < bricks[piece][rot].h; r++)
	{
		for (int c = 0; c < bricks[piece][rot].w; c++)
		{
			if (bricks[piece][rot].grid[r][c] == 'X')
			{
				rect.left = c*block_size + offset.x;
				rect.top = (rows-1)*block_size - (r*block_size + offset.y);
				rect.right = rect.left + block_size;
				rect.bottom = rect.top + block_size;
				FillRect(hdc, &rect, hbr1);

				rect.left++;
				rect.top++;
				rect.bottom--;
				rect.right--;
				FillRect(hdc, &rect, hbr2);
			}
		}
	}
	DeleteObject(hbr1);
	DeleteObject(hbr2);
}